(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5378],{59739:function(e,t,n){"use strict";var r=n(56669);function o(){}function i(){}i.resetWarningCache=o,e.exports=function(){function e(e,t,n,o,i,c){if(c!==r){var u=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw u.name="Invariant Violation",u}}function t(){return e}e.isRequired=e;var n={array:e,bigint:e,bool:e,func:e,number:e,object:e,string:e,symbol:e,any:e,arrayOf:t,element:e,elementType:e,instanceOf:t,node:e,objectOf:t,oneOf:t,oneOfType:t,shape:t,exact:t,checkPropTypes:i,resetWarningCache:o};return n.PropTypes=n,n}},47329:function(e,t,n){e.exports=n(59739)()},56669:function(e){"use strict";e.exports="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED"},55106:function(e,t,n){"use strict";n.d(t,{Z:function(){return O}});var r=n(82394),o=n(75582),i=n(82684),c=n(83455),u=n(9518),a=n(77011),s=n(28598);function l(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function p(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?l(Object(n),!0).forEach((function(t){(0,r.Z)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):l(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var f=u.default.div.withConfig({displayName:"MultiFileInput__DropzoneStyle",componentId:"sc-1l6yd2y-0"})(["&:hover{cursor:pointer;}"]);var d=function(e){var t=e.children,n=e.inputOnChange,r=e.inputProps,o=e.onDragActiveChange,c=e.setFiles,u=(0,i.useCallback)((function(e){c(e)}),[c]),l=(0,a.uI)({onDrop:u}),d=l.getInputProps,y=l.getRootProps,v=l.isDragActive,h=d(),O=h.accept,b=h.autoComplete,g=h.multiple,j=h.onChange,P=h.onClick,_=h.ref,w=h.style,m=h.tabIndex,C=h.type,D=p(p({},r),{},{accept:O,autoComplete:b,multiple:g,onChange:function(e){null===n||void 0===n||n(e),null===r||void 0===r||r.onChange(e),j(e)},onClick:P,ref:_,style:w,tabIndex:m,type:C});return(0,i.useEffect)((function(){null===o||void 0===o||o(v)}),[v,o]),(0,s.jsxs)(f,p(p({},y()),{},{children:[(0,s.jsx)("input",p(p({},D),{},{directory:"",webkitdirectory:""})),t]}))},y=n(82531);function v(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function h(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?v(Object(n),!0).forEach((function(t){(0,r.Z)(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):v(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var O=function(e){var t=e.children,n=e.directoryPath,u=e.onDragActiveChange,a=e.setFileUploadProgress,l=e.setUploadedFiles,p=(0,c.Db)(y.ZP.files.useCreate({onUploadProgress:function(e,t){var n,o=t.body,i=[null===o||void 0===o?void 0:o.dir_path,null===o||void 0===o||null===(n=o.file)||void 0===n?void 0:n.name].filter((function(e){return(null===e||void 0===e?void 0:e.length)>=1})).join("/");null===a||void 0===a||a((function(t){return h(h({},t),{},(0,r.Z)({},i,e.loaded/e.total))}))}})),f=(0,o.Z)(p,1)[0],v=(0,i.useCallback)((function(e){e.forEach((function(e){var t=e.name,o=e.path,i=[n],c=o.split("/").filter((function(e){return e&&e!==t})).join("/");c&&i.push(c);var u=[],s=i.join("/");(null===s||void 0===s?void 0:s.length)>=1&&i.push(s),u.push(t);var p=u.join("/");f({dir_path:s,file:e,overwrite:!1}).then((function(e){var t=e.data,n=t.error,o=t.file;l((function(e){return h(h({},e),{},(0,r.Z)({},p,o||n))}))})),a((function(e){return h(h({},e),{},(0,r.Z)({},p,0))}))}))}),[f,n,a,l]);return(0,s.jsx)(d,{onDragActiveChange:u,setFiles:v,children:t})}},50439:function(e,t,n){"use strict";n.r(t);var r=n(55106),o=n(28598);t.default=function(){return(0,o.jsx)("div",{children:(0,o.jsx)(r.Z,{})})}},91185:function(e,t,n){(window.__NEXT_P=window.__NEXT_P||[]).push(["/test",function(){return n(50439)}])}},function(e){e.O(0,[7011,9774,2888,179],(function(){return t=91185,e(e.s=t);var t}));var t=e.O();_N_E=t}]);